import { ApplicationRef, Component, OnInit } from '@angular/core';
import { AlertController, ModalController, PopoverController, ToastController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { CalendarEditorPopover } from 'src/app/components/calendar-editor-popover/calendar-editor.popover';
import { OptionsPopover } from 'src/app/components/options-popover/options.popover';
import { Calendar } from 'src/app/interfaces/calendar';
import { CalendarService } from 'src/app/services/calendar.service';

@Component({
  selector: 'app-calendars',
  templateUrl: './calendars.page.html',
  styleUrls: ['./calendars.page.scss'],
})
export class CalendarsPage implements OnInit {

  calendars: Calendar[] = [];
  text: string = "";

  private translation: { [key: string] : string; } = {
    'calendars.confirm_deletion_question': 'Are you sure you want to delete the selected calendar?',
    'commons.cancel': 'Cancel',
    'commons.confirm': 'Confirm',
    'commons.delete': 'Delete',
    'commons.edit': 'Edit',
  };

  constructor(
    private alertController: AlertController,
    private modalController: ModalController,
    private popover: PopoverController,
    private toastController: ToastController,
    private app: ApplicationRef,
    private calendarService: CalendarService,
    private translateService: TranslateService,
  ) {}

  ngOnInit() {
    this.loadCalendars();
    this.translateService.get(Object.keys(this.translation)).subscribe({
      next: translation => this.translation = translation
    });
  }

  async editCalendar(calendar?: Calendar) {
    const pop = await this.popover.create({
      component: CalendarEditorPopover,
      componentProps: {
        calendar: Object.assign({}, calendar)
      },
    });
    pop.onDidDismiss().then((event) => {
      if (event.role === 'save') {
        this.loadCalendars();
      } else if (event.role === 'error') {
        this.showToast(event.data);
      }
    });

    await pop.present();
  }

  async showCalendarOptions(event: Event, calendar: Calendar) {
    const pop = await this.popover.create({
      component: OptionsPopover,
      event: event,
      translucent: true,
    });
    pop.onDidDismiss().then((result) => {
      switch (result.role) {
        case "delete":
          this.confirmCalendarDeletion(calendar);
          break;
        case "edit":
          this.editCalendar(calendar);
          break;
      }
    });
    return await pop.present();
  }

  private async confirmCalendarDeletion(calendar: Calendar) {
    const alert = await this.alertController.create({
      header: this.translation['commons.confirm'],
      message: this.translation['calendars.confirm_deletion_question'],
      buttons: [
        {
          cssClass: 'secondary',
          role: 'cancel',
          text: this.translation['commons.cancel'],
        },
        {
          handler: () => this.deleteCalendar(calendar),
          role: 'submit',
          text: this.translation['commons.delete'],
        }
      ],
    });

    await alert.present();
  }

  private deleteCalendar(calendar: Calendar) {
    this.calendarService.deleteCalendar(calendar).subscribe({
      complete: () => this.loadCalendars(),
      error: (error) => this.showToast(error),
    });
  }

  private loadCalendars() {
    this.calendarService.getCalendars().subscribe({
      next: calendars => {
        this.calendars = calendars;
        this.app.tick();
      }
    });
  }

  private async showToast(messageKey: string) {
    const toast = await this.toastController.create({
      duration: 3000,
      message: this.translateService.instant(messageKey),
      position: 'top',
    });

    await toast.present();
  }

}
