import { ApplicationRef, Component, OnInit } from '@angular/core';
import { AlertController, PopoverController, ToastController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { CoworkerEditorPopover } from 'src/app/components/coworker-editor-popover/coworker-editor.popover';
import { OptionsPopover } from 'src/app/components/options-popover/options.popover';
import { Coworker } from 'src/app/models/coworker';
import { CoworkerService } from 'src/app/services/coworker.service';

@Component({
  selector: 'app-coworkers',
  templateUrl: './coworkers.page.html',
  styleUrls: ['./coworkers.page.scss'],
})
export class CoworkersPage implements OnInit {

  coworkers: Coworker[] = [];

  private translation: { [key: string] : string; } = {
    'coworkers.confirm_deletion_question': 'Are you sure you want to delete the selected coworker?',
    'commons.cancel': 'Cancel',
    'commons.confirm': 'Confirm',
    'commons.delete': 'Delete',
    'commons.edit': 'Edit',
  };

  constructor(
    private alertController: AlertController,
    private popover: PopoverController,
    private toastController: ToastController,
    private app: ApplicationRef,
    private coworkerService: CoworkerService,
    private translateService: TranslateService,
  ) {}

  ngOnInit() {
    this.translateService.get(Object.keys(this.translation)).subscribe({
      next: translation => this.translation = translation
    });
  }

  createCoworker() {
    this.editCoworker(new Coworker());
  }

  async editCoworker(coworker: Coworker) {
    const pop = await this.popover.create({
      component: CoworkerEditorPopover,
      componentProps: {
        coworker: Object.assign(new Coworker(), coworker)
      },
    });
    pop.onDidDismiss().then((event) => {
      if (event.role === 'save') {
        this.loadCoworkers();
      } else if (event.role === 'error') {
        this.showToast(event.data);
      }
    });

    await pop.present();
  }

  ionViewWillEnter() {
    this.loadCoworkers();
  }

  async showPopover(event: Event, coworker: Coworker) {
    const pop = await this.popover.create({
      component: OptionsPopover,
      event: event,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        switch (result.role) {
          case "delete":
            this.confirmCoworkerDeletion(coworker);
            break;
          case "edit":
            this.editCoworker(coworker);
            break;
        }
      }
    );
    return await pop.present();
  }

  private async confirmCoworkerDeletion(coworker: Coworker) {
    const alert = await this.alertController.create({
      header: this.translation['commons.confirm'],
      message: this.translation['coworkers.confirm_deletion_question'],
      buttons: [
        {
          cssClass: 'secondary',
          role: 'cancel',
          text: this.translation['commons.cancel'],
        },
        {
          handler: () => this.deleteCoworker(coworker),
          role: 'submit',
          text: this.translation['commons.delete'],
        }
      ],
    });

    await alert.present();
  }

  private deleteCoworker(coworker: Coworker) {
    this.coworkerService.deleteCoworker(coworker).subscribe({
      complete: () => this.loadCoworkers()
    });
  }

  private loadCoworkers() {
    this.coworkerService.getCoworkers().subscribe({
      next: coworkers => {
        this.coworkers = coworkers;
        this.app.tick();
      }
    });
  }

  private async showToast(messageKey: string) {
    const toast = await this.toastController.create({
      duration: 3000,
      message: this.translateService.instant(messageKey),
      position: 'top',
    });

    await toast.present();
  }

}
