import { WeekDay, DatePipe, getLocaleFirstDayOfWeek } from "@angular/common";
import { Component, ApplicationRef, LOCALE_ID, Inject } from "@angular/core";
import { PopoverController } from "@ionic/angular";
import { HomeEditPopover } from "src/app/components/home-edit-popover/home-edit.popover";
import { HomeViewPopover } from "src/app/components/home-view-popover/home-view.popover";
import { TurnCreationPopover } from "src/app/components/turn-creation-popover/turn-creation.popover";
import { TurnDeletionPopover } from "src/app/components/turn-deletion-popover/turn-deletion.popover";
import { TurnSwapPopover } from "src/app/components/turn-swap-popover/turn-swap.popover";
import { Week } from "src/app/models/week";
import { ShiftService } from "src/app/services/shift.service";

@Component({
  selector: 'app-home',
  templateUrl: 'home.page.html',
  styleUrls: ['home.page.scss'],
})
export class HomePage {

  locale: string = 'en';
  today: Date = new Date();
  month: Date;
  weeks: Week[] = [];

  private firstDayOfWeek = WeekDay.Sunday;

  constructor(
    private app: ApplicationRef,
    private popover: PopoverController,
    private shiftService: ShiftService,
    private datePipe: DatePipe,
  ) {
    if (window.Intl && typeof window.Intl === 'object') {
      this.locale = navigator.language;
      try {
        this.firstDayOfWeek = getLocaleFirstDayOfWeek(this.locale);
      } catch (error) {
      }
    }
    this.loadMonth(new Date(this.today.getFullYear(), this.today.getMonth(), 1));
  }

  loadPrevious() {
    this.loadMonth(new Date(this.month.getFullYear(), this.month.getMonth() - 1, 1));
  }

  loadNext() {
    this.loadMonth(new Date(this.month.getFullYear(), this.month.getMonth() + 1, 1));
  }

  async showEditPopover(event: Event) {
    const pop = await this.popover.create({
      component: HomeEditPopover,
      event: event,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        switch (result.role) {
          case "addTurn":
            this.showTurnCreationPopover();
            break;
          case "deleteTurn":
            this.showTurnDeletionPopover();
            break;
          case "swapTurn":
            this.showTurnSwapPopover();
            break;
        }
      }
    );
    return await pop.present();
  }

  private async showTurnCreationPopover() {
    const pop = await this.popover.create({
      component: TurnCreationPopover,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        if (result.role === "save") {
          this.loadShifts();
        }
      }
    );
    return await pop.present();
  };

  private async showTurnDeletionPopover() {
    const pop = await this.popover.create({
      component: TurnDeletionPopover,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        if (result.role === "delete") {
          this.loadShifts();
        }
      }
    );
    return await pop.present();
  };

  private async showTurnSwapPopover() {
    const pop = await this.popover.create({
      component: TurnSwapPopover,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        if (result.role === "save") {
          this.loadShifts();
        }
      }
    );
    return await pop.present();
  };

  async showViewPopover(event: Event) {
    const pop = await this.popover.create({
      component: HomeViewPopover,
      event: event,
      translucent: true,
    });
    return await pop.present();
  }

  private loadMonth(month: Date) {
    this.month = new Date(month.getFullYear(), month.getMonth(), 1);
    this.weeks = [];
    const firstDay = this.getFirstDay();
    for (let i = 0; i < 6; i++) {
      this.weeks.push(new Week(new Date(firstDay.getFullYear(), firstDay.getMonth(), firstDay.getDate() + i*7)));
    }
    this.loadShifts();
  }

  private getFirstDay(): Date {
    var diff = this.month.getDay() - this.firstDayOfWeek;
    if (diff < 0) {
      diff = diff + 7;
    }
    return new Date(this.month.getFullYear(), this.month.getMonth(), this.month.getDate() - diff);
  }

  private loadShifts() {
    this.weeks.forEach((week) => week.days.forEach((day) => day.shifts = []));
    const from = this.datePipe.transform(this.weeks[0].days[0].date, 'yyyy-MM-dd');
    const to = this.datePipe.transform(this.weeks[5].days[6].date, 'yyyy-MM-dd');
    this.shiftService.findShifts(from, to).subscribe({
      next: (shifts) => {
        shifts.forEach(
          (shift) => this.weeks.forEach(
            (week) => week.days.filter(
              (day) => this.datePipe.transform(day.date, 'yyyy-MM-dd') === shift.date
            ).forEach(
              (day) => day.shifts.push(shift)
            )
          )
        );
        this.app.tick();
      }
    });
  }

}
