import { ApplicationRef, Component, NgZone, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AlertController, PopoverController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { OptionsPopover } from 'src/app/components/options-popover/options.popover';
import { ShiftType } from 'src/app/models/shift-type';
import { ShiftTypeService } from 'src/app/services/shift-type.service';

@Component({
  selector: 'app-shift-types',
  templateUrl: './shift-types.page.html',
  styleUrls: ['./shift-types.page.scss'],
})
export class ShiftTypesPage implements OnInit {

  shiftTypes: ShiftType[] = [];

  private translation: { [key: string] : string; } = {
    'shift_types.confirm_deletion_question': 'Are you sure you want to delete the selected shift type?',
    'commons.cancel': 'Cancel',
    'commons.confirm': 'Confirm',
    'commons.delete': 'Delete',
    'commons.edit': 'Edit',
  };

  constructor(
    private alertController: AlertController,
    private popover: PopoverController,
    private app: ApplicationRef,
    private router: Router,
    private shiftTypeService: ShiftTypeService,
    private translateService: TranslateService,
    private ngZone: NgZone,
  ) {}

  ngOnInit() {
    this.translateService.get(Object.keys(this.translation)).subscribe({
      next: translation => this.translation = translation
    });
  }

  createShiftType() {
    this.ngZone.run(() => this.router.navigate(['shift-type-editor']));
  }

  ionViewWillEnter() {
    this.loadShiftTypes();
  }

  async showPopover(event: Event, shiftType: ShiftType) {
    const pop = await this.popover.create({
      component: OptionsPopover,
      event: event,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        switch (result.role) {
          case "delete":
            this.confirmShiftTypeDeletion(shiftType);
            break;
          case "edit":
            this.editShiftType(shiftType);
            break;
        }
      }
    );
    return await pop.present();
  }

  private async confirmShiftTypeDeletion(shiftType: ShiftType) {
    const alert = await this.alertController.create({
      header: this.translation['commons.confirm'],
      message: this.translation['shift_types.confirm_deletion_question'],
      buttons: [
        {
          cssClass: 'secondary',
          role: 'cancel',
          text: this.translation['commons.cancel'],
        },
        {
          handler: () => this.deleteShiftType(shiftType),
          role: 'submit',
          text: this.translation['commons.delete'],
        }
      ],
    });

    await alert.present();
  }

  private deleteShiftType(shiftType: ShiftType) {
    this.shiftTypeService.deleteShiftType(shiftType).subscribe({
      complete: () => this.loadShiftTypes()
    });
  }

  private editShiftType(shiftType: ShiftType) {
    this.ngZone.run(() => this.router.navigate(['shift-type-editor'], { state: { shiftType: shiftType } }));
  }

  private loadShiftTypes() {
    this.shiftTypeService.getShiftTypes().subscribe({
      next: shiftTypes => {
        this.shiftTypes = shiftTypes;
        this.app.tick();
      }
    });
  }

}
