import { DatePipe } from '@angular/common';
import { ApplicationRef, Component, OnInit } from '@angular/core';
import { PopoverController } from '@ionic/angular';
import { SwapsFilterPopover } from 'src/app/components/swaps-filter-popover/swaps-filter.popover';
import { TurnSwapPopover } from 'src/app/components/turn-swap-popover/turn-swap.popover';
import { Calendar } from 'src/app/interfaces/calendar';
import { Coworker } from 'src/app/models/coworker';
import { Swap } from 'src/app/models/swap';
import { ShiftService } from 'src/app/services/shift.service';

@Component({
  selector: 'app-swaps',
  templateUrl: './swaps.page.html',
  styleUrls: ['./swaps.page.scss'],
})
export class SwapsPage implements OnInit {

  private filterCoworker: Coworker = null;
  private filterCalendar: Calendar = null;
  private fromDate: string = null;
  private swaps: Swap[] = [];
  private toDate: string = null;

  constructor(
    private popover: PopoverController,
    private app: ApplicationRef,
    private shiftService: ShiftService,
    private datePipe: DatePipe,
  ) {}

  ngOnInit() {
  }

  async addSwap() {
    const pop = await this.popover.create({
      component: TurnSwapPopover,
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        if (result.role === "save") {
          this.loadSwaps();
        }
      }
    );
    return await pop.present();
  };

  async showFiltersPopover() {
    const pop = await this.popover.create({
      component: SwapsFilterPopover,
      componentProps: {
        filterCalendar: this.filterCalendar,
        filterCoworker: this.filterCoworker,
        fromDate: this.fromDate,
        toDate: this.toDate,
      },
      translucent: true,
    });
    pop.onDidDismiss().then(
      (result) => {
        if (result.role === "apply") {
          this.filterCalendar = result.data.filterCalendar;
          this.filterCoworker = result.data.filterCoworker;
          this.fromDate = result.data.fromDate;
          this.toDate = result.data.toDate;
        }
      }
    );
    return await pop.present();
  };

  calculateHours(): number {
    return this.calculateOwingHours() - this.calculateOwedHours();
  }

  calculateOwedHours(): number {
    return this.sumHours(this.getFromCoworkerSwaps());
  }

  calculateOwingHours(): number {
    return this.sumHours(this.getToCoworkerSwaps());
  }

  filterSwaps(): Swap[] {
    return this.swaps.filter((it) => this.matchFilters(it));
  }

  getFromCoworkerSwaps(): Swap[] {
    return this.filterSwaps().filter((it) => it.isFromCoworker());
  }

  getToCoworkerSwaps(): Swap[] {
    return this.filterSwaps().filter((it) => it.isToCoworker());
  }

  ionViewWillEnter() {
    this.loadSwaps();
  }

  private loadSwaps() {
    this.shiftService.findAllSwaps().subscribe({
      next: swaps => {
        this.swaps = swaps;
        this.app.tick();
      }
    });
  }

  private matchCalendar(swap: Swap): boolean {
    return this.filterCalendar == null || swap.shift.calendar.id === this.filterCalendar.id;
  }

  private matchCoworker(swap: Swap): boolean {
    return this.filterCoworker == null || swap.coworker.id === this.filterCoworker.id;
  }

  private matchFilters(swap: Swap): boolean {
    return this.matchFromDate(swap) && this.matchToDate(swap) && this.matchCalendar(swap) && this.matchCoworker(swap);
  }

  private matchFromDate(swap: Swap): boolean {
    
    return this.fromDate == null || swap.shift.date >= this.datePipe.transform(this.fromDate, 'yyyy-MM-dd');
  }

  private matchToDate(swap: Swap): boolean {
    return this.toDate == null || swap.shift.date <= this.datePipe.transform(this.toDate, 'yyyy-MM-dd');
  }

  private sumHours(swaps: Swap[]): number {
    return swaps.map((it) => it.shift.type.hours).reduce((prev, cur) => prev + cur, 0);
  }

}
