import { Injectable } from '@angular/core';
import { Observable, Subject, throwError } from 'rxjs';
import { Calendar } from '../interfaces/calendar';
import { Coworker } from '../models/coworker';
import { DatabaseService } from './database.service';

@Injectable({
  providedIn: 'root'
})
export class CoworkerService {

  constructor(private database: DatabaseService) {}

  deleteCoworker(coworker: Coworker): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction((tx) => {
        tx.executeSql('DELETE FROM coworkers WHERE id = ?', [coworker.id], null, null);
        tx.executeSql('DELETE FROM calendars_coworkers WHERE coworker_id = ?', [coworker.id], null, null);
      }).then(() => result.complete()).catch(() => result.error("coworkers.error.could_not_delete"))
    );
    return result;
  }

  getCoworkers(): Observable<Coworker[]> {
    const coworkers = new Subject<Coworker[]>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'SELECT * FROM coworkers',
          [],
          (tx, coworkersResults) => tx.executeSql('SELECT * FROM calendars_coworkers', [], (_, calendarsCoworkersResults) => coworkers.next(this.readCoworkers(coworkersResults, calendarsCoworkersResults)), (_, err) => coworkers.error(err)),
          (_, err) => coworkers.error(err)
        )
      )
    );
    return coworkers;
  }

  saveCoworker(coworker: Coworker): Observable<void> {
    coworker.name = coworker.name?.trim();
    if (!coworker.name) {
      return throwError("coworkers.error.empty_name");
    }
    if (!coworker.id) {
      return this.save(coworker);
    }
    return this.update(coworker);
  }

  private readCoworkers(coworkersResults, calendarsCoworkersResults): Coworker[] {
    const coworkersMap: Record<number, Coworker> = {};
    const calendarsMap: Record<number, Calendar> = {};
    for (let i = 0; i < coworkersResults.rows.length; i++) {
      const row = coworkersResults.rows.item(i);
      const coworker = Object.assign(new Coworker(), row);
      coworkersMap[coworker.id] = coworker;
    }
    for (let i = 0; i < calendarsCoworkersResults.rows.length; i++) {
      const row = calendarsCoworkersResults.rows.item(i);
      const coworker = coworkersMap[row['coworker_id']];
      let calendar = calendarsMap[row['calendar_id']];
      if (!calendar) {
        calendar = { id: row['calendar_id']} as Calendar;
        calendarsMap[calendar.id] = calendar;
      }
      coworker.calendars.push(calendar);
    }
    return Object.values(coworkersMap);
  }

  private save(coworker: Coworker): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'INSERT INTO coworkers (name) VALUES (?)',
          [coworker.name],
          (tx, insert) => coworker.calendars.forEach(calendar => {
            tx.executeSql('INSERT INTO calendars_coworkers (calendar_id, coworker_id) VALUES (?, ?)', [calendar.id, insert.insertId], null, () => result.error("coworkers.error.could_not_save"));
          }),
          () => result.error("coworkers.error.could_not_save")
        )
      ).then(() => result.complete()).catch(() => result.error("coworkers.error.could_not_save"))
    );
    return result;
  }

  private update(coworker: Coworker): Observable<any> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction((tx) => {
        tx.executeSql('UPDATE coworkers SET name = ? WHERE id = ?', [coworker.name, coworker.id], null, () => result.error("coworkers.error.could_not_update"));
        tx.executeSql('DELETE FROM calendars_coworkers WHERE coworker_id = ?', [coworker.id], null, () => result.error("coworkers.error.could_not_save"));
        coworker.calendars.forEach(calendar => {
          tx.executeSql('INSERT INTO calendars_coworkers (calendar_id, coworker_id) VALUES (?, ?)', [calendar.id, coworker.id], null, () => result.error("coworkers.error.could_not_save"));
        });
      }).then(() => result.complete()).catch(() => result.error("coworkers.error.could_not_save"))
    );
    return result;
  }

}
