import { Injectable } from '@angular/core';
import { Capacitor } from '@capacitor/core';
import { SQLite, SQLiteObject } from '@ionic-native/sqlite/ngx';
import { Platform } from '@ionic/angular';
import { AsyncSubject, Observable } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class DatabaseService {

  private database: AsyncSubject<SQLiteObject> = new AsyncSubject();

  constructor(
    private platform: Platform,
    private sqlite: SQLite,
  ) {
    this.platform.ready().then(() => {
      if (Capacitor.isPluginAvailable('SQLite')) {
        this.sqlite.create({
          name: 'turnreturn.db',
          location: 'default'
        }).then((database: SQLiteObject) => {
          this.initDatabase(database);
        });
      } else {
        this.initDatabase(new SQLiteObject((<any> window).openDatabase('turnreturn', '', 'TurnReturn offline database', 2 * 1024 * 1024)));
      }
    });
  }

  getDatabase(): Observable<SQLiteObject> {
    return this.database;
  }

  private initDatabase(database: SQLiteObject) {
    database.transaction((tx) => {
      tx.executeSql('CREATE TABLE IF NOT EXISTS calendars (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT NOT NULL UNIQUE);');
      tx.executeSql('INSERT OR IGNORE INTO calendars (id, name) VALUES (1, \'Hospital\');');
      tx.executeSql('INSERT OR IGNORE INTO calendars (id, name) VALUES (2, \'Nursing Home\');');
      tx.executeSql('CREATE TABLE IF NOT EXISTS shift_types (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT NOT NULL UNIQUE, initials TEXT NOT NULL UNIQUE, from_hour TEXT NOT NULL, to_hour TEXT NOT NULL, hours REAL, price REAL, color TEXT);');
      tx.executeSql('INSERT OR IGNORE INTO shift_types (id, name, initials, from_hour, to_hour, hours, price, color) VALUES (1, \'Morning\', \'M\', \'08:00\', \'15:00\', 7, 0, \'#E67C73\');');
      tx.executeSql('INSERT OR IGNORE INTO shift_types (id, name, initials, from_hour, to_hour, hours, price, color) VALUES (2, \'Afternoon\', \'A\', \'15:00\', \'22:00\', 7, 0, \'#33B679\');');
      tx.executeSql('INSERT OR IGNORE INTO shift_types (id, name, initials, from_hour, to_hour, hours, price, color) VALUES (3, \'Night\', \'N\', \'22:00\', \'08:00\', 10, 0, \'#3F51B5\');');
      tx.executeSql('CREATE TABLE IF NOT EXISTS calendars_shift_types (calendar_id INTEGER NOT NULL, shift_type_id INTEGER NOT NULL, FOREIGN KEY(calendar_id) REFERENCES calendars(id), FOREIGN KEY(shift_type_id) REFERENCES shift_types(id), PRIMARY KEY (calendar_id, shift_type_id));');
      tx.executeSql('INSERT OR IGNORE INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (1, 1);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (1, 2);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (1, 3);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (2, 1);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (2, 2);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (2, 3);');
      tx.executeSql('CREATE TABLE IF NOT EXISTS shifts (id INTEGER PRIMARY KEY AUTOINCREMENT, date TEXT NOT NULL, calendar_id INTEGER NOT NULL, shift_type_id INTEGER NOT NULL, FOREIGN KEY(calendar_id) REFERENCES calendars(id), FOREIGN KEY(shift_type_id) REFERENCES shift_types(id));');
      tx.executeSql('CREATE TABLE IF NOT EXISTS coworkers (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT NOT NULL UNIQUE);');
      tx.executeSql('INSERT OR IGNORE INTO coworkers (id, name) VALUES (1, \'John Doe\');');
      tx.executeSql('INSERT OR IGNORE INTO coworkers (id, name) VALUES (2, \'Jane Doe\');');
      tx.executeSql('CREATE TABLE IF NOT EXISTS calendars_coworkers (calendar_id INTEGER NOT NULL, coworker_id INTEGER NOT NULL, FOREIGN KEY(calendar_id) REFERENCES calendars(id), FOREIGN KEY(coworker_id) REFERENCES coworkers(id), PRIMARY KEY (calendar_id, coworker_id));');
      tx.executeSql('INSERT OR IGNORE INTO calendars_coworkers (calendar_id, coworker_id) VALUES (1, 1);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_coworkers (calendar_id, coworker_id) VALUES (1, 2);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_coworkers (calendar_id, coworker_id) VALUES (2, 1);');
      tx.executeSql('INSERT OR IGNORE INTO calendars_coworkers (calendar_id, coworker_id) VALUES (2, 2);');
      tx.executeSql('CREATE TABLE IF NOT EXISTS swaps (shift_id INTEGER PRIMARY KEY NOT NULL, coworker_id INTEGER NOT NULL, direction TEXT NOT NULL, FOREIGN KEY(shift_id) REFERENCES shifts(id), FOREIGN KEY(coworker_id) REFERENCES coworkers(id));');
      this.database.next(database);
      this.database.complete();
    });
  }

}
