import { Injectable } from '@angular/core';
import { Observable, Subject, throwError } from 'rxjs';
import { Calendar } from '../interfaces/calendar';
import { ShiftType } from '../models/shift-type';
import { DatabaseService } from './database.service';

@Injectable({
  providedIn: 'root'
})
export class ShiftTypeService {

  constructor(private database: DatabaseService) {}

  deleteShiftType(shiftType: ShiftType): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction((tx) => {
        tx.executeSql('DELETE FROM shift_types WHERE id = ?', [shiftType.id], null, null);
        tx.executeSql('DELETE FROM calendars_shift_types WHERE shift_type_id = ?', [shiftType.id], null, null);
      }).then(() => result.complete()).catch(() => result.error("shift_types.error.could_not_delete"))
    );
    return result;
  }

  getShiftTypes(): Observable<ShiftType[]> {
    const shiftTypes = new Subject<ShiftType[]>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'SELECT id, name, initials, from_hour as fromHour, to_hour as toHour, hours, price, color FROM shift_types',
          [],
          (tx, shiftTypesResults) => tx.executeSql('SELECT * FROM calendars_shift_types', [], (_, calendarsShiftTypesResults) => shiftTypes.next(this.readShiftTypes(shiftTypesResults, calendarsShiftTypesResults)), (_, err) => shiftTypes.error(err)),
          (_, err) => shiftTypes.error(err)
        )
      )
    );
    return shiftTypes;
  }

  saveShiftType(shiftType: ShiftType): Observable<void> {
    shiftType.name = shiftType.name?.trim();
    if (!shiftType.name) {
      return throwError("shift_types.error.empty_name");
    }
    if (!shiftType.id) {
      return this.save(shiftType);
    }
    return this.update(shiftType);
  }

  private readShiftTypes(shiftTypesResults, calendarsShiftTypesResults): ShiftType[] {
    const shiftTypesMap: Record<number, ShiftType> = {};
    const calendarsMap: Record<number, Calendar> = {};
    for (let i = 0; i < shiftTypesResults.rows.length; i++) {
      const row = shiftTypesResults.rows.item(i);
      const shiftType = Object.assign(new ShiftType(), row);
      shiftTypesMap[shiftType.id] = shiftType;
    }
    for (let i = 0; i < calendarsShiftTypesResults.rows.length; i++) {
      const row = calendarsShiftTypesResults.rows.item(i);
      const shiftType = shiftTypesMap[row['shift_type_id']];
      let calendar = calendarsMap[row['calendar_id']];
      if (!calendar) {
        calendar = { id: row['calendar_id']} as Calendar;
        calendarsMap[calendar.id] = calendar;
      }
      shiftType.calendars.push(calendar);
    }
    return Object.values(shiftTypesMap);
  }

  private save(shiftType: ShiftType): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'INSERT INTO shift_types (name, initials, from_hour, to_hour, hours, price, color) VALUES (?, ?, ?, ?, ?, ?, ?)',
          [shiftType.name, shiftType.initials, shiftType.fromHour, shiftType.toHour, shiftType.hours, shiftType.price, shiftType.color],
          (tx, insert) => shiftType.calendars.forEach(calendar => {
            tx.executeSql('INSERT INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (?, ?)', [calendar.id, insert.insertId], null, () => result.error("shift_types.error.could_not_save"));
          }),
          () => result.error("shift_types.error.could_not_save")
        )
      ).then(() => result.complete()).catch(() => result.error("shift_types.error.could_not_save"))
    );
    return result;
  }

  private update(shiftType: ShiftType): Observable<any> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction((tx) => {
        tx.executeSql(
          'UPDATE shift_types SET name = ?, initials = ?, from_hour = ?, to_hour = ?, hours = ?, price = ?, color = ? WHERE id = ?',
          [shiftType.name, shiftType.initials, shiftType.fromHour, shiftType.toHour, shiftType.hours, shiftType.price, shiftType.color, shiftType.id],
          null,
          () => result.error("shift_types.error.could_not_update")
        );
        tx.executeSql('DELETE FROM calendars_shift_types WHERE shift_type_id = ?', [shiftType.id], null, () => result.error("shift_types.error.could_not_save"));
        shiftType.calendars.forEach(calendar => {
          tx.executeSql('INSERT INTO calendars_shift_types (calendar_id, shift_type_id) VALUES (?, ?)', [calendar.id, shiftType.id], null, () => result.error("shift_types.error.could_not_save"));
        });
      }).then(() => result.complete()).catch(() => result.error("shift_types.error.could_not_save"))
    );
    return result;
  }

}
