import { Injectable } from '@angular/core';
import { Observable, Subject, throwError } from 'rxjs';
import { Calendar } from '../interfaces/calendar';
import { Coworker } from '../models/coworker';
import { Shift } from '../models/shift';
import { ShiftType } from '../models/shift-type';
import { Swap } from '../models/swap';
import { DatabaseService } from './database.service';

@Injectable({
  providedIn: 'root'
})
export class ShiftService {

  constructor(private database: DatabaseService) {}

  deleteShift(shift: Shift): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction((tx) => {
        tx.executeSql('DELETE FROM swaps WHERE shift_id = ?', [shift.id], null, null);
        tx.executeSql('DELETE FROM shifts WHERE id = ?', [shift.id], null, null);
      }).then(() => result.complete()).catch(() => result.error("shifts.error.could_not_delete"))
    );
    return result;
  }

  findShifts(from: String, to: String): Observable<Shift[]> {
    const shifts = new Subject<Shift[]>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'SELECT * FROM shifts sh LEFT JOIN swaps sw ON sw.shift_id = sh.id WHERE sh.date >= ? AND sh.date <= ?',
          [from, to],
          (tx, shiftResults) => tx.executeSql(
            'SELECT id, name, initials, from_hour as fromHour, to_hour as toHour, hours, price, color FROM shift_types',
            [],
            (_, shiftTypesResults) => shifts.next(this.readShifts(shiftResults, shiftTypesResults)),
            (_, err) => shifts.error(err)
          ),
          (_, err) => shifts.error(err)
        )
      )
    );
    return shifts;
  }

  findAllShiftsByCalendarAndDate(calendar: Calendar, date: String): Observable<Shift[]> {
    const shifts = new Subject<Shift[]>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'SELECT * FROM shifts sh LEFT JOIN swaps sw ON sw.shift_id = sh.id WHERE sh.calendar_id = ? AND sh.date = ?',
          [calendar.id, date],
          (tx, shiftResults) => tx.executeSql(
            'SELECT id, name, initials, from_hour as fromHour, to_hour as toHour, hours, price, color FROM shift_types',
            [],
            (_, shiftTypeResults) => shifts.next(this.readShifts(shiftResults, shiftTypeResults)),
            (_, err) => shifts.error(err)
          ),
          (_, err) => shifts.error(err)
        )
      )
    );
    return shifts;
  }

  findShiftsByCalendarAndDate(calendar: Calendar, date: String): Observable<Shift[]> {
    const shifts = new Subject<Shift[]>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'SELECT * FROM shifts WHERE calendar_id = ? AND date = ? AND ID NOT IN (SELECT shift_id FROM swaps)',
          [calendar.id, date],
          (tx, shiftResults) => tx.executeSql(
            'SELECT id, name, initials, from_hour as fromHour, to_hour as toHour, hours, price, color FROM shift_types',
            [],
            (_, shiftTypeResults) => shifts.next(this.readShifts(shiftResults, shiftTypeResults)),
            (_, err) => shifts.error(err)
          ),
          (_, err) => shifts.error(err)
        )
      )
    );
    return shifts;
  }

  findAllSwaps(): Observable<Swap[]> {
    const swaps = new Subject<Swap[]>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'SELECT * FROM swaps sw LEFT JOIN shifts sh ON sh.id = sw.shift_id',
          [],
          (tx, swapResults) => tx.executeSql(
            'SELECT id, name, initials, from_hour as fromHour, to_hour as toHour, hours, price, color FROM shift_types',
            [],
            (tx, shiftTypeResults) => tx.executeSql(
              'SELECT * FROM coworkers',
              [],
              (_, coworkerResults) => swaps.next(this.readSwaps(swapResults, shiftTypeResults, coworkerResults)),
              (_, err) => swaps.error(err)
            ),
            (_, err) => swaps.error(err)
          ),
          (_, err) => swaps.error(err)
        )
      )
    );
    return swaps;
  }

  saveShift(shift: Shift): Observable<void> {
    if (!shift.date || !shift.calendar.id || !shift.type.id) {
      return throwError("shifts.error.could_not_save");
    }
    if (!shift.id) {
      return this.save(shift);
    }
    return this.update(shift);
  }

  private readShifts(shiftResults, shiftTypeResults): Shift[] {
    const calendarsMap: Record<number, Calendar> = {};
    const shiftTypesMap: Record<number, ShiftType> = {};
    const shifts: Shift[] = [];
    for (let i = 0; i < shiftTypeResults.rows.length; i++) {
      const row = shiftTypeResults.rows.item(i);
      const shiftType = Object.assign(new ShiftType(), row);
      shiftTypesMap[shiftType.id] = shiftType;
    }
    for (let i = 0; i < shiftResults.rows.length; i++) {
      const row = shiftResults.rows.item(i);
      const shift: Shift = Object.assign(new Shift(), row)
      let calendar: Calendar = calendarsMap[row['calendar_id']];
      if (!calendar) {
        calendar = { id: row['calendar_id']} as Calendar;
        calendarsMap[calendar.id] = calendar;
      }
      shift.calendar = calendar;
      shift.type = shiftTypesMap[row['shift_type_id']];
      if (row['shift_id']) {
        shift.swap = new Swap();
        shift.swap.shift = shift;
        shift.swap.coworker = new Coworker();
        shift.swap.coworker.id = row['coworker_id'];
        shift.swap.direction = row['direction'];
      }
      shifts.push(shift);
    }
    return shifts;
  }

  private readSwaps(swapResults, shiftTypeResults, coworkerResults): Swap[] {
    const calendarsMap: Record<number, Calendar> = {};
    const shiftTypesMap: Record<number, ShiftType> = {};
    const coworkersMap: Record<number, Coworker> = {};
    const swaps: Swap[] = [];
    for (let i = 0; i < shiftTypeResults.rows.length; i++) {
      const row = shiftTypeResults.rows.item(i);
      const shiftType = Object.assign(new ShiftType(), row);
      shiftTypesMap[shiftType.id] = shiftType;
    }
    for (let i = 0; i < coworkerResults.rows.length; i++) {
      const row = coworkerResults.rows.item(i);
      const coworker = Object.assign(new Coworker(), row);
      coworkersMap[coworker.id] = coworker;
    }
    for (let i = 0; i < swapResults.rows.length; i++) {
      const row = swapResults.rows.item(i);
      const swap: Swap = Object.assign(new Swap(), row)
      const shift: Shift = Object.assign(new Shift(), row)
      let calendar: Calendar = calendarsMap[row['calendar_id']];
      if (!calendar) {
        calendar = { id: row['calendar_id']} as Calendar;
        calendarsMap[calendar.id] = calendar;
      }
      shift.calendar = calendar;
      shift.type = shiftTypesMap[row['shift_type_id']];
      shift.swap = swap;
      swap.shift = shift;
      swap.coworker = coworkersMap[row['coworker_id']];
      swaps.push(swap);
    }
    return swaps;
  }

  private save(shift: Shift): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'INSERT INTO shifts (date, calendar_id, shift_type_id) VALUES (?, ?, ?)',
          [shift.date, shift.calendar.id, shift.type.id],
          (tx, insert) => {
            if (shift.swap != null) {
              tx.executeSql('INSERT INTO swaps (shift_id, coworker_id, direction) VALUES (?, ?, ?)', [insert.insertId, shift.swap.coworker.id, shift.swap.direction]);
            }
          },
        )
      ).then(() => result.complete()).catch(() => result.error("shifts.error.could_not_save"))
    );
    return result;
  }

  private update(shift: Shift): Observable<void> {
    const result = new Subject<void>();
    this.database.getDatabase().subscribe(
      (db) => db.transaction(
        (tx) => tx.executeSql(
          'DELETE FROM swaps WHERE shift_id = ?',
          [shift.id],
          (tx) => {
            if (shift.swap != null) {
              tx.executeSql('INSERT INTO swaps (shift_id, coworker_id, direction) VALUES (?, ?, ?)', [shift.id, shift.swap.coworker.id, shift.swap.direction]);
            }
          },
        )
      ).then(() => result.complete()).catch(() => result.error("shifts.error.could_not_save"))
    );
    return result;
  }

}
